#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2010 Richard Dymond (rjdymond@gmail.com)
#
# This file is part of SkoolKit.
#
# SkoolKit is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# SkoolKit is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# SkoolKit. If not, see <http://www.gnu.org/licenses/>.

import sys
from os.path import abspath, dirname, join
sys.path.insert(0, join(abspath(dirname(__file__)), 'lib'))
from skoolparser import SkoolParser

BLOCKS = 'b'
BLOCK_TITLES = 't'
BLOCK_DESC = 'd'
REGISTERS = 'r'
BLOCK_COMMENTS = 'm'
SUBBLOCKS = 's'
COMMENTS = 'c'

def write_line(text):
    sys.stdout.write('%s\n' % text)

def get_instruction_ctl(op):
    ctl = 'c'
    if op.startswith('DEFB'):
        ctl = 'b'
    elif op.startswith('DEFW'):
        ctl = 'w'
    elif op.startswith('DEFM'):
        ctl = 't'
    elif op.startswith('DEFS'):
        ctl = 'z'
    return ctl

def write_body(entry, elements):
    # Split the entry by mid-routine comments
    instructions = entry.instructions
    blocks = [('', [instructions[0]])]
    for instruction in instructions[1:]:
        mrc = instruction.get_mid_routine_comment()
        if mrc:
            blocks.append((mrc, [instruction]))
        else:
            blocks[-1][1].append(instruction)

    for mrc, instructions in blocks:
        if BLOCK_COMMENTS in elements and mrc:
            write_line('D %s %s' % (instructions[0].address, mrc))

        if SUBBLOCKS not in elements:
            continue

        # Split the block by comment rowspans and instruction type
        sub_blocks = []
        i = 0
        prev_ctl = ''
        while i < len(instructions):
            instruction = instructions[i]
            comment = instruction.comment
            ctl = get_instruction_ctl(instruction.operation)
            if comment.rowspan > 1 or comment.text:
                sub_blocks.append((ctl, instructions[i:i + comment.rowspan]))
                prev_ctl = ''
            elif sub_blocks and ctl == prev_ctl:
                sub_blocks[-1][1].append(instruction)
            else:
                sub_blocks.append((ctl, [instruction]))
                prev_ctl = ctl
            i += comment.rowspan

        # Write a ctl line for each sub-block
        for j, (ctl, instructions) in enumerate(sub_blocks):
            length = 0
            if j + 1 < len(sub_blocks):
                length = int(sub_blocks[j + 1][1][0].address) - int(instructions[0].address)
            comment = instructions[0].comment.text if COMMENTS in elements else ''
            parent_ctl = 'b' if entry.ctl in 'gu' else entry.ctl
            if ctl == parent_ctl:
                # Don't write a ctl line for a commentless sub-block of the
                # same type as the parent block
                if not comment:
                    continue
                block_ctl = ' '
            else:
                block_ctl = ctl.upper()
            address = instructions[0].address
            if length:
                write_line(('%s %s,%i %s' % (block_ctl, address, length, comment)).rstrip())
            else:
                write_line(('%s %s %s' % (block_ctl, address, comment)).rstrip())

def write_entry(entry, elements):
    if BLOCKS in elements:
        if BLOCK_TITLES in elements:
            write_line(('%s %s %s' % (entry.ctl, entry.address, entry.description)).rstrip())
        else:
            write_line('%s %s' % (entry.ctl, entry.address))
    if BLOCK_DESC in elements:
        if entry.details:
            write_line('D %s %s' % (entry.address, entry.details))
    if REGISTERS in elements:
        for reg in entry.registers:
            write_line('R %s %s %s' % (entry.address, reg.name, reg.contents))
    write_body(entry, elements)
    if BLOCK_COMMENTS in elements and entry.end_comment:
        write_line('E %s %s' % (entry.address, entry.end_comment))

def write_ctls(entries, elements):
    for entry in entries:
        write_entry(entry, elements)

def parse_options(options):
    elements = BLOCKS + BLOCK_TITLES + BLOCK_DESC + REGISTERS + BLOCK_COMMENTS + SUBBLOCKS + COMMENTS
    i = 0
    while i < len(options):
        option = options[i]
        if option[:2] == '-w':
            elements = option[2:]
        i += 1
    return elements

################################################################################
# Begin
################################################################################
if len(sys.argv) < 2:
    sys.stderr.write('Usage: %s [options] <skoolfile>\n' % sys.argv[0])
    sys.stderr.write('\nOptions:\n')
    sys.stderr.write("  -wX  Write only these elements, where X is one or more of:\n")
    sys.stderr.write("         %s = block types and addresses\n" % BLOCKS)
    sys.stderr.write("         %s = block titles\n" % BLOCK_TITLES)
    sys.stderr.write("         %s = block descriptions\n" % BLOCK_DESC)
    sys.stderr.write("         %s = registers\n" % REGISTERS)
    sys.stderr.write("         %s = mid-block comments and block end comments\n" % BLOCK_COMMENTS)
    sys.stderr.write("         %s = sub-block types and addresses\n" % SUBBLOCKS)
    sys.stderr.write("         %s = instruction-level comments\n" % COMMENTS)
    sys.exit(1)

elements = parse_options(sys.argv[1:-1])

# Create the parser
skoolfile = sys.argv[-1]
mode = 0
verbose = False
warn = False
fix_mode = 0
html = False
create_labels = False
mode_args = (mode, verbose, warn, fix_mode, html, create_labels)
parser = SkoolParser(skoolfile, mode_args)

# Write the ctl directives
write_ctls(parser.memory_map, elements)
