#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2009, 2010 Richard Dymond (rjdymond@gmail.com)
#
# This file is part of SkoolKit.
#
# SkoolKit is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# SkoolKit is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# SkoolKit. If not, see <http://www.gnu.org/licenses/>.

import sys
from os.path import abspath, dirname, join, isfile
sys.path.insert(0, join(abspath(dirname(__file__)), 'lib'))
from snaskool import SkoolWriter, Recipe, get_snapshot, parse_ctl, generate_ctl

def parse_options(options):
    ctlfname = '%s.ctl' % snafile[:-4]
    genctl = False
    start = 16384
    text = False
    defb_size = 8
    defb_mod = 1
    zfill = False
    asm = False
    i = 0
    while i < len(options):
        option = options[i]
        if option == '-c':
            ctlfname = options[i + 1]
            i += 1
        elif option == '-g':
            genctl = True
            ctlfname = options[i + 1]
            i += 1
        elif option == '-s':
            start = int(options[i + 1])
            i += 1
        elif option == '-t':
            text = True
        elif option == '-n':
            defb_size = int(options[i + 1])
            i += 1
        elif option == '-m':
            defb_mod = int(options[i + 1])
            i += 1
        elif option == '-z':
            zfill = True
        elif option == '-a':
            asm = True
        i += 1
    return ctlfname, genctl, start, text, defb_size, defb_mod, zfill, asm

###############################################################################
# Begin
###############################################################################

# Parse options
if len(sys.argv) < 2:
    sys.stderr.write('Usage: %s [options] file.[sna|z80]\n\n' % sys.argv[0])
    sys.stderr.write('Options:\n')
    sys.stderr.write('  -c FILE  Use FILE as the ctl file (default is file.ctl)\n')
    sys.stderr.write('  -g FILE  Generate a control file in FILE\n')
    sys.stderr.write('  -s ADDR  Specify the address at which to start disassembling\n')
    sys.stderr.write('  -t       Show ASCII text in the comment fields\n')
    sys.stderr.write("  -n N     Set the max number of bytes per DEFB statement to N (default=8)\n")
    sys.stderr.write("  -m M     Group DEFB blocks by addresses that divide M\n")
    sys.stderr.write("  -z       Write bytes with leading zeroes in DEFB statements\n")
    sys.stderr.write('  -a       Output plain ASM\n')
    sys.exit(1)

snafile = sys.argv[-1]
ctlfile, genctl, start, text, defb_size, defb_mod, zfill, asm = parse_options(sys.argv[1:-1])

# Read the snapshot file
snapshot = get_snapshot(snafile)

# Pad out the end of the snapshot to avoid errors when an instruction crosses
# the 64K boundary
snapshot.extend((0, 0, 0))

if genctl:
    # Analyse the snapshot and generate a ctl file...
    recipe = Recipe(generate_ctl(snapshot, ctlfile, start))
elif isfile(ctlfile):
    # ...or read the existing ctl file...
    recipe = parse_ctl(ctlfile)
else:
    # ...or use a default one
    recipe = Recipe({start: 'c'})

# Print the skool/ASM file
writer = SkoolWriter(snapshot, recipe, defb_size, defb_mod, zfill)
writer.write_skool(text, asm)
